// ----------------------------------------------------------------------
// C source file that prints "Hello Character World!" 
// using the character based vga display.
//
// Eric Li, Ke Xu, Winston Chao
// ----------------------------------------------------------------------
#include "xbasic_types.h"
#include "xio.h"

#include "xintc_l.h"
#include "xuartlite_l.h"

#define W 80
#define H 60
#define VGA_START 0x00800000
#define RED 0xE0
#define GREEN 0x1C
#define BLUE 0x03

// defined in isr.c
extern void uart_handler(void *callback);
extern int uart_interrupt_count;
extern char uart_character;

/*
 * setup_interrupts: Initialize the interrupt sources and handlers
 *
 * Should be called once when the system starts
 *
 * The main _interrupt_handler() function from Xilinx
 *
 * Saves and restores CPU context, etc.
 *
 * Sees which interrupts are pending, and for each it
 *     acknowledges the interrupt and
 *     calls a user-defined interrupt handler in Xintc_InterruptVectorTable
 *
 * Place interrupt service routines in isr.c to ensure they are placed in
 * the proper memory segment.
 */
void setup_interrupts()
{
  /*
   * Reset the interrupt controller peripheral
   */

  /* Disable the interrupt signal */
  XIntc_mMasterDisable(XPAR_INTC_SINGLE_BASEADDR);

  /* Disable all interrupt sources */
  XIntc_mEnableIntr(XPAR_INTC_SINGLE_BASEADDR,0);

  /* Acknowledge all possible interrupt sources
     to make sure none are pending */
  XIntc_mAckIntr(XPAR_INTC_SINGLE_BASEADDR, 0xffffffff);

  /*
   * Install the UART interrupt handler
   */

  XIntc_InterruptVectorTable[XPAR_INTC_MYUART_INTERRUPT_INTR].Handler =
    uart_handler;

  /*
   * Enable interrupt sources
   */

  /* Enable CPU interrupts */
  microblaze_enable_interrupts();

  /* Enable interrupts from the interrupt controller */
  XIntc_mMasterEnable(XPAR_INTC_SINGLE_BASEADDR);

  /* Tell the interrupt controller to accept interrupts from the UART */
  XIntc_mEnableIntr(XPAR_INTC_SINGLE_BASEADDR, XPAR_MYUART_INTERRUPT_MASK);
  
  /* Enable UART interrupt generation */
  XUartLite_mEnableIntr(XPAR_MYUART_BASEADDR);
}

void clear_screen()
{
  int x, y;

  // erasing screen
  for(x=0 ; x<H*W ; x++)
    XIo_Out8(VGA_START + x, 0);

  // draw the border smileys
  for ( x = 0; x < W; x++ )
    XIo_Out8(VGA_START + x, 1);
  for ( x = 0; x < W; x++ )
    XIo_Out8(VGA_START + x + 59*W, 1);
  for ( y = 0; y < H; y++ )
    XIo_Out8(VGA_START + 0 + y*W, 1);
  for ( y = 0; y < H; y++ )
    XIo_Out8(VGA_START + 79 + y*W, 1);
    
  // draw the text
  XIo_Out8(VGA_START + 2027, 'H');
  XIo_Out8(VGA_START + 2028, 'e');
  XIo_Out8(VGA_START + 2029, 'l');
  XIo_Out8(VGA_START + 2030, 'l');
  XIo_Out8(VGA_START + 2031, 'o');

  XIo_Out8(VGA_START + 2033, 'C');
  XIo_Out8(VGA_START + 2034, 'h');
  XIo_Out8(VGA_START + 2035, 'a');
  XIo_Out8(VGA_START + 2036, 'r');
  XIo_Out8(VGA_START + 2037, 'a');
  XIo_Out8(VGA_START + 2038, 'c');
  XIo_Out8(VGA_START + 2039, 't');
  XIo_Out8(VGA_START + 2040, 'e');
  XIo_Out8(VGA_START + 2041, 'r');
  
  XIo_Out8(VGA_START + 2043, 'W');
  XIo_Out8(VGA_START + 2044, 'o');
  XIo_Out8(VGA_START + 2045, 'r');
  XIo_Out8(VGA_START + 2046, 'l');
  XIo_Out8(VGA_START + 2047, 'd');
  XIo_Out8(VGA_START + 2048, '!');

}

int main()
{
  char c;
  int i;
  char buf[2];
  buf[1] = '\0';

  // Enable the instruction cache: makes the code run 6 times faster
  microblaze_enable_icache();

  setup_interrupts();

  clear_screen();

  return 0;
}
